package com.digiwin.athena.app.service.basic;

import org.apache.commons.codec.binary.Hex;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.crypto.Cipher;
import javax.crypto.SecretKey;
import javax.crypto.spec.IvParameterSpec;
import javax.crypto.spec.SecretKeySpec;
import java.nio.charset.StandardCharsets;
import java.util.Base64;

public class AESUtils {


    private static final Logger logger = LoggerFactory.getLogger(AESUtils.class);
    private static final String IV_STRING = "ghUb#er57HBh(u%g";

    /**
     * 加密
     * 加密失败返回原文 2021-7-21
     *
     * @param src    加密字段
     * @param aesKey aesKey 长度16
     * @return 密文 string
     */
    public static String aesEncryptByBase64(String src, String aesKey) {
        try {
            SecretKeySpec key = new SecretKeySpec(aesKey.getBytes(StandardCharsets.UTF_8), "AES");

            byte[] initParam = IV_STRING.getBytes(StandardCharsets.UTF_8);
            IvParameterSpec ivParameterSpec = new IvParameterSpec(initParam);
            Cipher cipher = Cipher.getInstance("AES/CBC/PKCS5Padding");
            cipher.init(Cipher.ENCRYPT_MODE, key, ivParameterSpec);

            byte[] cleartext = src.getBytes(StandardCharsets.UTF_8);
            byte[] ciphertextBytes = cipher.doFinal(cleartext);
            Base64.Encoder encoder = Base64.getEncoder();
            return encoder.encodeToString(ciphertextBytes);
        } catch (Exception ex) {
            logger.error("AES加密失败[{}]", src);
            return src;
        }

    }

    public static String aesEncrypt(String src, String aesKey) {
        try {
            // 生成和mysql一致的加密数据
            SecretKeySpec key = generateMySQLAESKey(aesKey);
            Cipher cipher = Cipher.getInstance("AES/ECB/PKCS5Padding");
            cipher.init(Cipher.ENCRYPT_MODE, key);

            byte[] cleartext = src.getBytes(StandardCharsets.UTF_8);
            byte[] ciphertextBytes = cipher.doFinal(cleartext);
            return new String(Hex.encodeHexString(ciphertextBytes)).toUpperCase();
        } catch (Exception ex) {
            logger.error("AES加密失败[{}]", src);
            return src;
        }
    }

    public static String aesDecrypt(String content, String aesKey) {
        try {
            SecretKey key = generateMySQLAESKey(aesKey);
            Cipher cipher = Cipher.getInstance("AES");
            cipher.init(Cipher.DECRYPT_MODE, key);
            byte[] cleartext = Hex.decodeHex(content.toCharArray());
            byte[] ciphertextBytes = cipher.doFinal(cleartext);
            return new String(ciphertextBytes, StandardCharsets.UTF_8);
        } catch (Exception ex) {
            logger.error("AES解密失败[{}]", content);
            return content;
        }
    }

    public static SecretKeySpec generateMySQLAESKey(final String key) {
        final byte[] finalKey = new byte[16];
        int i = 0;
        for (byte b : key.getBytes(StandardCharsets.UTF_8)) {
            finalKey[i++ % 16] ^= b;
        }
        return new SecretKeySpec(finalKey, "AES");
    }
}
